@@ -65,6 +65,9 @@
 #define PAS_ID_PREFIX "pas-id-"
 #define SUMMARY_FLUSH_TIMEOUT 5000
 
+#define EDB_ERROR(_code) e_data_book_create_error (E_DATA_BOOK_STATUS_ ## _code, NULL)
+#define EDB_ERROR_EX(_code, _msg) e_data_book_create_error (E_DATA_BOOK_STATUS_ ## _code, _msg)
+
 G_DEFINE_TYPE (EBookBackendFile, e_book_backend_file, E_TYPE_BOOK_BACKEND_SYNC)
 
 struct _EBookBackendFilePrivate {
@@ -87,23 +90,29 @@ static struct {
 	DB_ENV *env;
 } global_env;
 
-static EBookBackendSyncStatus
-db_error_to_status (const gint db_error)
+static void
+db_error_to_gerror (const gint db_error, GError **perror)
 {
+	if (db_error && perror && *perror)
+		g_clear_error (perror);
+
 	switch (db_error) {
 	case 0:
-		return GNOME_Evolution_Addressbook_Success;
+		return;
 	case DB_NOTFOUND:
-		return GNOME_Evolution_Addressbook_ContactNotFound;
+		g_propagate_error (perror, EDB_ERROR (CONTACT_NOT_FOUND));
+		return;
 	case EACCES:
-		return GNOME_Evolution_Addressbook_PermissionDenied;
+		g_propagate_error (perror, EDB_ERROR (PERMISSION_DENIED));
+		return;
 	default:
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "db error 0x%x (%s)", db_error, db_strerror (db_error) ? db_strerror (db_error) : "Unknown error"));
+		return;
 	}
 }
 
 static void
-string_to_dbt(const gchar *str, DBT *dbt)
+string_to_dbt (const gchar *str, DBT *dbt)
 {
 	memset (dbt, 0, sizeof (*dbt));
 	dbt->data = (gpointer)str;
@@ -138,19 +147,19 @@ build_summary (EBookBackendFilePrivate *bfpriv)
 
 	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
 	memset (&id_dbt, 0, sizeof (id_dbt));
-	db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+	db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
 	while (db_error == 0) {
 
 		/* don't include the version in the list of cards */
-		if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
+		if (id_dbt.size != strlen (E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
 		    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 			EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
 			e_book_backend_summary_add_contact (bfpriv->summary, contact);
 			g_object_unref (contact);
 		}
 
-		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+		db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 
 	}
 
@@ -176,7 +185,7 @@ set_revision (EContact *contact)
 	const struct tm *tm = NULL;
 	time_t t;
 
-	t = time(NULL);
+	t = time (NULL);
 	tm = gmtime (&t);
 	if (tm)
 		strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
@@ -184,10 +193,11 @@ set_revision (EContact *contact)
 
 }
 
-static EBookBackendSyncStatus
-do_create(EBookBackendFile  *bf,
+static gboolean
+do_create (EBookBackendFile  *bf,
 	  const gchar      *vcard_req,
-	  EContact **contact)
+	  EContact **contact,
+	  GError **perror)
 {
 	DB             *db = bf->priv->file_db;
 	DBT            id_dbt, vcard_dbt;
@@ -230,32 +240,33 @@ do_create(EBookBackendFile  *bf,
 	}
 
 	g_free (id);
-	return db_error_to_status (db_error);
+	db_error_to_gerror (db_error, perror);
+
+	return db_error == 0;
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_create_contact (EBookBackendSync *backend,
 				    EDataBook *book,
 				    guint32 opid,
 				    const gchar *vcard,
-				    EContact **contact)
+				    EContact **contact,
+				    GError **perror)
 {
-	EBookBackendSyncStatus status;
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 
-	status = do_create (bf, vcard, contact);
-	if (status == GNOME_Evolution_Addressbook_Success) {
+	if (do_create (bf, vcard, contact, perror)) {
 		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
 	}
-	return status;
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 				     EDataBook *book,
 				     guint32 opid,
 				     GList *id_list,
-				     GList **ids)
+				     GList **ids,
+				     GError **perror)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	DB             *db = bf->priv->file_db;
@@ -264,7 +275,6 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 	gchar          *id;
 	GList         *l;
 	GList         *removed_cards = NULL;
-	GNOME_Evolution_Addressbook_CallStatus rv = GNOME_Evolution_Addressbook_Success;
 
 	for (l = id_list; l; l = l->next) {
 		id = l->data;
@@ -274,7 +284,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 		db_error = db->del (db, NULL, &id_dbt, 0);
 		if (0 != db_error) {
 			g_warning (G_STRLOC ": db->del failed with %s", db_strerror (db_error));
-			rv = db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
 			continue;
 		}
 
@@ -294,16 +304,15 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 		gchar *id = l->data;
 		e_book_backend_summary_remove_contact (bf->priv->summary, id);
 	}
-
-	return rv;
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_modify_contact (EBookBackendSync *backend,
 				    EDataBook *book,
 				    guint32 opid,
 				    const gchar *vcard,
-				    EContact **contact)
+				    EContact **contact,
+				    GError **perror)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	DB             *db = bf->priv->file_db;
@@ -315,8 +324,10 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	*contact = e_contact_new_from_vcard (vcard);
 	id = e_contact_get_const (*contact, E_CONTACT_UID);
 
-	if (id == NULL)
-		return GNOME_Evolution_Addressbook_OtherError;
+	if (id == NULL) {
+		g_propagate_error (perror, EDB_ERROR_EX (OTHER_ERROR, "No UID in the contact"));
+		return;
+	}
 
 	/* update the revision (modified time of contact) */
 	set_revision (*contact);
@@ -351,18 +362,16 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	}
 	g_free (vcard_with_rev);
 
-	if (0 == db_error)
-		return GNOME_Evolution_Addressbook_Success;
-	else
-		return db_error_to_status (db_error);
+	db_error_to_gerror (db_error, perror);
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_get_contact (EBookBackendSync *backend,
 				 EDataBook *book,
 				 guint32 opid,
 				 const gchar *id,
-				 gchar **vcard)
+				 gchar **vcard,
+				 GError **perror)
 {
 	EBookBackendFile *bf;
 	DB             *db;
@@ -380,20 +389,21 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 
 	if (db_error == 0) {
 		*vcard = vcard_dbt.data;
-		return GNOME_Evolution_Addressbook_Success;
 	} else {
 		g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 		*vcard = g_strdup ("");
-		return GNOME_Evolution_Addressbook_ContactNotFound;
+
+		g_propagate_error (perror, EDB_ERROR (CONTACT_NOT_FOUND));
 	}
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 				      EDataBook *book,
 				      guint32 opid,
 				      const gchar *query,
-				      GList **contacts)
+				      GList **contacts,
+				      GError **perror)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	DB             *db = bf->priv->file_db;
@@ -404,20 +414,20 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 	gboolean search_needed;
 	const gchar *search = query;
 	GList *contact_list = NULL;
-	EBookBackendSyncStatus status;
 
 	d(printf ("e_book_backend_file_get_contact_list (%s)\n", search));
-	status = GNOME_Evolution_Addressbook_Success;
 	if (e_book_backend_summary_is_summary_query (bf->priv->summary, search)) {
 
 		/* do a summary query */
 		GPtrArray *ids = e_book_backend_summary_search (bf->priv->summary, search);
 		gint i;
 
-		if (!ids)
-			return GNOME_Evolution_Addressbook_ContactNotFound;
+		if (!ids) {
+			g_propagate_error (perror, EDB_ERROR (CONTACT_NOT_FOUND));
+			return;
+		}
 
-		for (i = 0; i < ids->len; i ++) {
+		for (i = 0; i < ids->len; i++) {
 			gchar *id = g_ptr_array_index (ids, i);
 			string_to_dbt (id, &id_dbt);
 			memset (&vcard_dbt, 0, sizeof (vcard_dbt));
@@ -428,7 +438,7 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 				contact_list = g_list_prepend (contact_list, vcard_dbt.data);
 			} else {
 				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
-				status = db_error_to_status (db_error);
+				db_error_to_gerror (db_error, perror);
 				break;
 			}
 		}
@@ -440,8 +450,8 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 
 		card_sexp = e_book_backend_sexp_new (search);
 		if (!card_sexp) {
-			/* XXX this needs to be an invalid query error of some sort*/
-			return GNOME_Evolution_Addressbook_OtherError;
+			g_propagate_error (perror, EDB_ERROR (INVALID_QUERY));
+			return;
 		}
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
@@ -449,18 +459,19 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		if (db_error != 0) {
 			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 			/* XXX this needs to be some CouldNotOpen error */
-			return db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
+			return;
 		}
 
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
 		vcard_dbt.flags = DB_DBT_MALLOC;
 		memset (&id_dbt, 0, sizeof (id_dbt));
-		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+		db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
 		while (db_error == 0) {
 
 			/* don't include the version in the list of cards */
-			if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
+			if (id_dbt.size != strlen (E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 
 				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
@@ -468,26 +479,25 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 				}
 			}
 
-			db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+			db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 
 		}
 		g_object_unref (card_sexp);
 
 		if (db_error == DB_NOTFOUND) {
-			status = GNOME_Evolution_Addressbook_Success;
+			/* Success */
 		} else {
 			g_warning (G_STRLOC ": dbc->c_get failed with %s", db_strerror (db_error));
-			status = db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
 		}
 
-		db_error = dbc->c_close(dbc);
+		db_error = dbc->c_close (dbc);
 		if (db_error != 0) {
 			g_warning (G_STRLOC ": dbc->c_close failed with %s", db_strerror (db_error));
 		}
 	}
 
 	*contacts = contact_list;
-	return status;
 }
 
 typedef struct {
@@ -556,7 +566,7 @@ book_view_thread (gpointer data)
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
 
-	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")")) {
+	if ( !strcmp (query, "(contains \"x-evolution-any-field\" \"\")")) {
 		e_data_book_view_notify_status_message (book_view, _("Loading..."));
 		allcontacts = TRUE;
 	} else {
@@ -575,7 +585,7 @@ book_view_thread (gpointer data)
 		if (!ids)
 			goto done;
 
-		for (i = 0; i < ids->len; i ++) {
+		for (i = 0; i < ids->len; i++) {
 			gchar *id = g_ptr_array_index (ids, i);
 
 			if (!e_flag_is_set (closure->running))
@@ -608,7 +618,7 @@ book_view_thread (gpointer data)
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error == 0) {
 
-			db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+			db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 			while (db_error == 0) {
 
 				if (!e_flag_is_set (closure->running))
@@ -624,7 +634,7 @@ book_view_thread (gpointer data)
 					g_free (vcard_dbt.data);
 				}
 
-				db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+				db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 			}
 
 			dbc->c_close (dbc);
@@ -641,7 +651,7 @@ book_view_thread (gpointer data)
 	}
 done:
 	if (e_flag_is_set (closure->running))
-		e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
+		e_data_book_view_notify_complete (book_view, NULL /* Success */);
 
 	/* unref the */
 	printf("book_view file uref \n");
@@ -731,12 +741,13 @@ e_book_backend_file_changes_foreach_key (const gchar *key, gpointer user_data)
 	}
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_get_changes (EBookBackendSync *backend,
 				 EDataBook *book,
 				 guint32 opid,
 				 const gchar *change_id,
-				 GList **changes_out)
+				 GList **changes_out,
+				 GError **perror)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	gint     db_error = 0;
@@ -748,7 +759,6 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	DBC *dbc;
 	GList *changes = NULL;
 	EBookBackendFileChangeContext ctx;
-	EBookBackendSyncStatus result;
 
 	memset (&id_dbt, 0, sizeof (id_dbt));
 	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
@@ -767,12 +777,12 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	if (db_error != 0) {
 		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 	} else {
-		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+		db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
 		while (db_error == 0) {
 
 			/* don't include the version in the list of cards */
-			if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
+			if (id_dbt.size != strlen (E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 				EContact *contact;
 				gchar *id = id_dbt.data;
@@ -793,20 +803,20 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 				/* check what type of change has occurred, if any */
 				switch (e_dbhash_compare (ehash, id, vcard_string)) {
 				case E_DBHASH_STATUS_SAME:
-					g_free(vcard_string);
+					g_free (vcard_string);
 					break;
 				case E_DBHASH_STATUS_NOT_FOUND:
 					ctx.add_cards = g_list_append (ctx.add_cards, vcard_string);
-					ctx.add_ids = g_list_append (ctx.add_ids, g_strdup(id));
+					ctx.add_ids = g_list_append (ctx.add_ids, g_strdup (id));
 					break;
 				case E_DBHASH_STATUS_DIFFERENT:
 					ctx.mod_cards = g_list_append (ctx.mod_cards, vcard_string);
-					ctx.mod_ids = g_list_append (ctx.mod_ids, g_strdup(id));
+					ctx.mod_ids = g_list_append (ctx.mod_ids, g_strdup (id));
 					break;
 				}
 			}
 
-			db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+			db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 		}
 		dbc->c_close (dbc);
 	}
@@ -817,7 +827,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	if (db_error != DB_NOTFOUND) {
 		g_warning ("e_book_backend_file_changes: error building list\n");
 		*changes_out = NULL;
-		result = db_error_to_status (db_error);
+		db_error_to_gerror (db_error, perror);
 	}
 	else {
 		/* Update the hash and build our changes list */
@@ -857,63 +867,88 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 
 		e_dbhash_write (ehash);
 
-		result = GNOME_Evolution_Addressbook_Success;
 		*changes_out = changes;
 	}
 
 	e_dbhash_destroy (ehash);
-
-	return GNOME_Evolution_Addressbook_Success;
 }
 
 static gchar *
-e_book_backend_file_extract_path_from_uri (const gchar *uri)
+e_book_backend_file_extract_path_from_source (ESource *source)
 {
-	g_assert (g_ascii_strncasecmp (uri, "file://", 7) == 0);
+	const gchar *user_data_dir;
+	const gchar *source_dir;
+	gchar *mangled_source_dir;
+	gchar *filename;
 
-	return g_filename_from_uri (uri, NULL, NULL);
+	user_data_dir = e_get_user_data_dir ();
+	source_dir = e_source_peek_relative_uri (source);
+
+	if (!source_dir || !g_str_equal (source_dir, "system"))
+		source_dir = e_source_peek_uid (source);
+
+	/* Mangle the URI to not contain invalid characters. */
+	mangled_source_dir = g_strdelimit (g_strdup (source_dir), ":/", '_');
+
+	filename = g_build_filename (
+		user_data_dir, "addressbook", mangled_source_dir, NULL);
+
+	g_free (mangled_source_dir);
+
+	return filename;
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_authenticate_user (EBookBackendSync *backend,
 				       EDataBook *book,
 				       guint32 opid,
 				       const gchar *user,
 				       const gchar *passwd,
-				       const gchar *auth_method)
+				       const gchar *auth_method,
+				       GError **perror)
 {
-	return GNOME_Evolution_Addressbook_Success;
+	/* Success */
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_get_required_fields (EBookBackendSync *backend,
 					  EDataBook *book,
 					  guint32 opid,
-					  GList **fields_out)
+					  GList **fields_out,
+					  GError **perror)
 {
 	GList *fields = NULL;
 
-	fields = g_list_append (fields , g_strdup(e_contact_field_name (E_CONTACT_FILE_AS)));
+	fields = g_list_append (fields , g_strdup (e_contact_field_name (E_CONTACT_FILE_AS)));
 	*fields_out = fields;
-	return GNOME_Evolution_Addressbook_Success;
 }
 
-static EBookBackendSyncStatus
+static void
+e_book_backend_file_get_supported_auth_methods (EBookBackendSync *backend,
+						EDataBook *book,
+						guint32 opid,
+						GList **methods_out,
+						GError **perror)
+{
+	*methods_out = NULL;
+}
+
+static void
 e_book_backend_file_get_supported_fields (EBookBackendSync *backend,
 					  EDataBook *book,
 					  guint32 opid,
-					  GList **fields_out)
+					  GList **fields_out,
+					  GError **perror)
 {
 	GList *fields = NULL;
 	gint i;
 
 	/* XXX we need a way to say "we support everything", since the
 	   file backend does */
-	for (i = 1; i < E_CONTACT_FIELD_LAST; i ++)
+	for (i = 1; i < E_CONTACT_FIELD_LAST; i++)
 		fields = g_list_append (fields, g_strdup (e_contact_field_name (i)));
 
 	*fields_out = fields;
-	return GNOME_Evolution_Addressbook_Success;
 }
 
 /*
@@ -958,10 +993,10 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, gchar *old_version)
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
 
-		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+		db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
 		while (db_error == 0) {
-			if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
+			if (id_dbt.size != strlen (E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 				EContact *contact;
 
@@ -993,9 +1028,11 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, gchar *old_version)
 				g_object_unref (contact);
 			}
 
-			db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+			db_error = dbc->c_get (dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 		}
 
+		dbc->c_close (dbc);
+
 		if (card_failed) {
 			g_warning ("failed to update %d cards", card_failed);
 			return FALSE;
@@ -1057,10 +1094,11 @@ file_errcall (const gchar *buf1, gchar *buf2)
 	g_warning ("libdb error: %s", buf2);
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
+static void
 e_book_backend_file_load_source (EBookBackend           *backend,
 				 ESource                *source,
-				 gboolean                only_if_exists)
+				 gboolean                only_if_exists,
+				 GError                **perror)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	gchar           *dirname, *filename;
@@ -1070,20 +1108,17 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	DB_ENV *env;
 	time_t db_mtime;
 	struct stat sb;
-	gchar *uri;
 
-	uri = e_source_get_uri (source);
-
-	dirname = e_book_backend_file_extract_path_from_uri (uri);
+	dirname = e_book_backend_file_extract_path_from_source (source);
 	filename = g_build_filename (dirname, "addressbook.db", NULL);
-	g_free (uri);
 
 	db_error = e_db3_utils_maybe_recover (filename);
 	if (db_error != 0) {
 		g_warning ("db recovery failed with %s", db_strerror (db_error));
 		g_free (dirname);
 		g_free (filename);
-		return db_error_to_status (db_error);
+		db_error_to_gerror (db_error, perror);
+		return;
 	}
 
 	G_LOCK (global_env);
@@ -1097,7 +1132,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			G_UNLOCK (global_env);
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
+			return;
 		}
 
 		env->set_errcall (env, file_errcall);
@@ -1109,12 +1145,13 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 		db_error = (*env->open) (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
 		if (db_error != 0) {
-			env->close(env, 0);
+			env->close (env, 0);
 			g_warning ("db_env_open failed with %s", db_strerror (db_error));
 			G_UNLOCK (global_env);
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
+			return;
 		}
 
 		global_env.env = env;
@@ -1129,7 +1166,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		g_warning ("db_create failed with %s", db_strerror (db_error));
 		g_free (dirname);
 		g_free (filename);
-		return db_error_to_status (db_error);
+		db_error_to_gerror (db_error, perror);
+		return;
 	}
 
 	db_error = (*db->open) (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1141,7 +1179,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			g_warning ("db format upgrade failed with %s", db_strerror (db_error));
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
+			return;
 		}
 
 		db->close (db, 0);
@@ -1150,7 +1189,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			g_warning ("db_create failed with %s", db_strerror (db_error));
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
+			return;
 		}
 
 		db_error = (*db->open) (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1165,7 +1205,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			g_warning ("db_create failed with %s", db_strerror (db_error));
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			db_error_to_gerror (db_error, perror);
+			return;
 		}
 
 		db_error = (*db->open) (db, NULL, filename, NULL, DB_HASH, DB_RDONLY | DB_THREAD, 0666);
@@ -1176,15 +1217,16 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			/* the database didn't exist, so we create the
 			   directory then the .db */
 			db->close (db, 0);
-			rv = g_mkdir_with_parents (dirname, 0777);
+			rv = g_mkdir_with_parents (dirname, 0700);
 			if (rv == -1 && errno != EEXIST) {
 				g_warning ("failed to make directory %s: %s", dirname, g_strerror (errno));
+				if (errno == EACCES || errno == EPERM)
+					g_propagate_error (perror, EDB_ERROR (PERMISSION_DENIED));
+				else
+					g_propagate_error (perror, e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "Failed to make directory %s: %s", dirname, g_strerror (errno)));
 				g_free (dirname);
 				g_free (filename);
-				if (errno == EACCES || errno == EPERM)
-					return GNOME_Evolution_Addressbook_PermissionDenied;
-				else
-					return GNOME_Evolution_Addressbook_OtherError;
+				return;
 			}
 
 			db_error = db_create (&db, env, 0);
@@ -1192,7 +1234,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 				g_warning ("db_create failed with %s", db_strerror (db_error));
 				g_free (dirname);
 				g_free (filename);
-				return db_error_to_status (db_error);
+				db_error_to_gerror (db_error, perror);
+				return;
 			}
 
 			db_error = (*db->open) (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
@@ -1203,11 +1246,9 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 			else {
 #ifdef CREATE_DEFAULT_VCARD
 				EContact *contact = NULL;
-				EBookBackendSyncStatus status;
 
-				status = do_create (bf, XIMIAN_VCARD, &contact);
-				if (status != GNOME_Evolution_Addressbook_Success)
-					g_warning ("Cannot create default contact: %d", status);
+				if (!do_create (bf, XIMIAN_VCARD, &contact, NULL))
+					g_warning ("Cannot create default contact");
 				if (contact)
 					g_object_unref (contact);
 #endif
@@ -1223,16 +1264,17 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		bf->priv->file_db = NULL;
 		g_free (dirname);
 		g_free (filename);
-		return db_error_to_status (db_error);
+		db_error_to_gerror (db_error, perror);
+		return;
 	}
 
 	if (!e_book_backend_file_maybe_upgrade_db (bf)) {
 		db->close (db, 0);
 		bf->priv->file_db = NULL;
-		g_warning ("e_book_backend_file_maybe_upgrade_db failed");
 		g_free (dirname);
 		g_free (filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, EDB_ERROR_EX (OTHER_ERROR, "e_book_backend_file_maybe_upgrade_db failed"));
+		return;
 	}
 
 	g_free (bf->priv->dirname);
@@ -1243,8 +1285,8 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	if (g_stat (bf->priv->filename, &sb) == -1) {
 		db->close (db, 0);
 		bf->priv->file_db = NULL;
-		g_warning ("stat(%s) failed", bf->priv->filename);
-		return GNOME_Evolution_Addressbook_OtherError;
+		g_propagate_error (perror, e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "stat(%s) failed", bf->priv->filename));
+		return;
 	}
 	db_mtime = sb.st_mtime;
 
@@ -1255,14 +1297,13 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	if (e_book_backend_summary_is_up_to_date (bf->priv->summary, db_mtime) == FALSE
 	    || e_book_backend_summary_load (bf->priv->summary) == FALSE ) {
 		if (!bf->priv->summary || !build_summary (bf->priv)) {
-			g_warning ("Failed to build summary for an address book %s", bf->priv->filename);
-			return GNOME_Evolution_Addressbook_OtherError;
+			g_propagate_error (perror, e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "Failed to build summary for an address book %s", bf->priv->filename));
+			return;
 		}
 	}
 
 	e_book_backend_set_is_loaded (backend, TRUE);
 	e_book_backend_set_is_writable (backend, writable);
-	return GNOME_Evolution_Addressbook_Success;
 }
 
 static gboolean
@@ -1283,19 +1324,22 @@ select_changes (const gchar *name)
 	return TRUE;
 }
 
-static EBookBackendSyncStatus
+static void
 e_book_backend_file_remove (EBookBackendSync *backend,
 			    EDataBook        *book,
-			    guint32           opid)
+			    guint32           opid,
+			    GError          **perror)
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	GDir *dir;
 
 	if (-1 == g_unlink (bf->priv->filename)) {
-		if (errno == EACCES || errno == EPERM)
-			return GNOME_Evolution_Addressbook_PermissionDenied;
-		else
-			return GNOME_Evolution_Addressbook_OtherError;
+		if (errno == EACCES || errno == EPERM) {
+			g_propagate_error (perror, EDB_ERROR (PERMISSION_DENIED));
+		} else {
+			g_propagate_error (perror, e_data_book_create_error_fmt (E_DATA_BOOK_STATUS_OTHER_ERROR, "Failed to remove file '%s': %s", bf->priv->filename, g_strerror (errno)));
+		}
+		return;
 	}
 
 	/* unref the summary before we remove the file so it's not written out again */
@@ -1329,7 +1373,6 @@ e_book_backend_file_remove (EBookBackendSync *backend,
 	   it here..  the only time we should return failure is if we
 	   failed to remove the actual data.  a failure should mean
 	   that the addressbook is still valid */
-	return GNOME_Evolution_Addressbook_Success;
 }
 
 static gchar *
@@ -1338,13 +1381,14 @@ e_book_backend_file_get_static_capabilities (EBookBackend *backend)
 	return g_strdup("local,do-initial-query,bulk-removes,contact-lists");
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
-e_book_backend_file_cancel_operation (EBookBackend *backend, EDataBook *book)
-{
-	return GNOME_Evolution_Addressbook_CouldNotCancel;
-}
 static void
-e_book_backend_file_set_mode (EBookBackend *backend,  GNOME_Evolution_Addressbook_BookMode mode)
+e_book_backend_file_cancel_operation (EBookBackend *backend, EDataBook *book, GError **perror)
+{
+	g_propagate_error (perror, EDB_ERROR (COULD_NOT_CANCEL));
+}
+
+static void
+e_book_backend_file_set_mode (EBookBackend *backend, EDataBookMode mode)
 {
 	if (e_book_backend_is_loaded (backend)) {
 		e_book_backend_notify_writable (backend, TRUE);
@@ -1367,35 +1411,13 @@ e_book_backend_file_sync (EBookBackend *backend)
 	}
 }
 
-static gboolean
-e_book_backend_file_construct (EBookBackendFile *backend)
-{
-	g_assert (backend != NULL);
-	g_assert (E_IS_BOOK_BACKEND_FILE (backend));
-
-	if (! e_book_backend_construct (E_BOOK_BACKEND (backend)))
-		return FALSE;
-
-	return TRUE;
-}
-
 /**
  * e_book_backend_file_new:
  */
 EBookBackend *
 e_book_backend_file_new (void)
 {
-	EBookBackendFile *backend;
-
-	backend = g_object_new (E_TYPE_BOOK_BACKEND_FILE, NULL);
-
-	if (! e_book_backend_file_construct (backend)) {
-		g_object_unref (backend);
-
-		return NULL;
-	}
-
-	return E_BOOK_BACKEND (backend);
+	return g_object_new (E_TYPE_BOOK_BACKEND_FILE, NULL);
 }
 
 static void
@@ -1497,23 +1519,24 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	backend_class = E_BOOK_BACKEND_CLASS (klass);
 
 	/* Set the virtual methods. */
-	backend_class->load_source             = e_book_backend_file_load_source;
-	backend_class->get_static_capabilities = e_book_backend_file_get_static_capabilities;
-	backend_class->start_book_view         = e_book_backend_file_start_book_view;
-	backend_class->stop_book_view          = e_book_backend_file_stop_book_view;
-	backend_class->cancel_operation        = e_book_backend_file_cancel_operation;
-	backend_class->set_mode                = e_book_backend_file_set_mode;
-	backend_class->sync                    = e_book_backend_file_sync;
-	sync_class->remove_sync                = e_book_backend_file_remove;
-	sync_class->create_contact_sync        = e_book_backend_file_create_contact;
-	sync_class->remove_contacts_sync       = e_book_backend_file_remove_contacts;
-	sync_class->modify_contact_sync        = e_book_backend_file_modify_contact;
-	sync_class->get_contact_sync           = e_book_backend_file_get_contact;
-	sync_class->get_contact_list_sync      = e_book_backend_file_get_contact_list;
-	sync_class->get_changes_sync           = e_book_backend_file_get_changes;
-	sync_class->authenticate_user_sync     = e_book_backend_file_authenticate_user;
-	sync_class->get_supported_fields_sync  = e_book_backend_file_get_supported_fields;
-	sync_class->get_required_fields_sync   = e_book_backend_file_get_required_fields;
+	backend_class->load_source			= e_book_backend_file_load_source;
+	backend_class->get_static_capabilities		= e_book_backend_file_get_static_capabilities;
+	backend_class->start_book_view			= e_book_backend_file_start_book_view;
+	backend_class->stop_book_view			= e_book_backend_file_stop_book_view;
+	backend_class->cancel_operation			= e_book_backend_file_cancel_operation;
+	backend_class->set_mode				= e_book_backend_file_set_mode;
+	backend_class->sync				= e_book_backend_file_sync;
+	sync_class->remove_sync				= e_book_backend_file_remove;
+	sync_class->create_contact_sync			= e_book_backend_file_create_contact;
+	sync_class->remove_contacts_sync		= e_book_backend_file_remove_contacts;
+	sync_class->modify_contact_sync			= e_book_backend_file_modify_contact;
+	sync_class->get_contact_sync			= e_book_backend_file_get_contact;
+	sync_class->get_contact_list_sync		= e_book_backend_file_get_contact_list;
+	sync_class->get_changes_sync			= e_book_backend_file_get_changes;
+	sync_class->authenticate_user_sync		= e_book_backend_file_authenticate_user;
+	sync_class->get_supported_auth_methods_sync	= e_book_backend_file_get_supported_auth_methods;
+	sync_class->get_supported_fields_sync		= e_book_backend_file_get_supported_fields;
+	sync_class->get_required_fields_sync		= e_book_backend_file_get_required_fields;
 
 	object_class->dispose = e_book_backend_file_dispose;
 	object_class->finalize = e_book_backend_file_finalize;
